@@ -27,9 +27,9 @@ import os
 from pysollib.gamedb import GI
 from pysollib.help import help_about, help_html
 from pysollib.mfxutil import Struct, openURL
-from pysollib.mfxutil import print_err
+from pysollib.mfxutil import USE_PIL, print_err
 from pysollib.mygettext import _
-from pysollib.pysolrandom import constructRandom
+from pysollib.pysolrandom import construct_random
 from pysollib.pysoltk import AllGames_StatsDialog, SingleGame_StatsDialog
 from pysollib.pysoltk import ColorsDialog
 from pysollib.pysoltk import EditTextDialog
@@ -44,10 +44,11 @@ from pysollib.pysoltk import PysolMenubarTk, PysolToolbarTk
 from pysollib.pysoltk import Status_StatsDialog, Top_StatsDialog
 from pysollib.pysoltk import TimeoutsDialog
 from pysollib.pysoltk import create_find_card_dialog
+from pysollib.pysoltk import create_full_picture_dialog
 from pysollib.pysoltk import create_solver_dialog
 from pysollib.settings import DEBUG
 from pysollib.settings import PACKAGE_URL, TITLE
-from pysollib.settings import TOP_TITLE
+from pysollib.settings import TOP_SIZE
 from pysollib.stats import FileStatsFormatter
 
 
@@ -65,6 +66,10 @@ class PysolMenubar(PysolMenubarTk):
             save=0,
             save_as=0,
             hold_and_quit=0,
+            rndplayed=0,
+            rndwon=0,
+            rndnotwon=0,
+            rndnotplayed=0,
             undo=0,
             redo=0,
             restart=0,
@@ -77,7 +82,9 @@ class PysolMenubar(PysolMenubarTk):
             quickplay=0,
             demo=0,
             highlight_piles=0,
+            autoscale=0,
             find_card=0,
+            full_picture=0,
             rules=0,
             pause=0,
             custom_game=0,
@@ -122,6 +129,10 @@ class PysolMenubar(PysolMenubarTk):
         opt = self.app.opt
         ms = self.menustate
         # 0 = DISABLED, 1 = ENABLED
+        ms.rndplayed = len(self._mGetPossibleRandomGames("played")) > 0
+        ms.rndwon = len(self._mGetPossibleRandomGames("won")) > 0
+        ms.rndnotwon = len(self._mGetPossibleRandomGames("not won")) > 0
+        ms.rndnotplayed = len(self._mGetPossibleRandomGames("not played")) > 0
         ms.save_as = game.canSaveGame()
         ms.hold_and_quit = ms.save_as
         if game.filename and ms.save_as:
@@ -151,8 +162,12 @@ class PysolMenubar(PysolMenubarTk):
             ms.quickplay = 1
         if opt.highlight_piles and game.getHighlightPilesStacks():
             ms.highlight_piles = 1
+        if opt.auto_scale:
+            ms.autoscale = 1
         if game.canFindCard():
             ms.find_card = 1
+        if game.canShowFullPicture():
+            ms.full_picture = 1
         if game.app.getGameRulesFilename(game.id):  # note: this may return ""
             ms.rules = 1
         if not game.finished:
@@ -171,6 +186,12 @@ class PysolMenubar(PysolMenubarTk):
         # File menu
         self.setMenuState(ms.save, "file.save")
         self.setMenuState(ms.save_as, "file.saveas")
+        self.setMenuState(ms.rndplayed, "file.selectrandomgame.gamesplayed")
+        self.setMenuState(ms.rndwon, "file.selectrandomgame.gamesplayedandwon")
+        self.setMenuState(ms.rndnotwon,
+                          "file.selectrandomgame.gamesplayedandnotwon")
+        self.setMenuState(ms.rndnotplayed,
+                          "file.selectrandomgame.gamesnotplayed")
         self.setMenuState(ms.hold_and_quit, "file.holdandquit")
         # Edit menu
         self.setMenuState(ms.undo, "edit.undo")
@@ -179,6 +200,7 @@ class PysolMenubar(PysolMenubarTk):
         self.updateBookmarkMenuState()
         self.setMenuState(ms.restart, "edit.restart")
         self.setMenuState(ms.custom_game, "edit.editcurrentgame")
+        self.setMenuState(ms.custom_game, "edit.deletecurrentgame")
         # Game menu
         self.setMenuState(ms.deal, "game.dealcards")
         self.setMenuState(ms.autodrop, "game.autodrop")
@@ -188,6 +210,7 @@ class PysolMenubar(PysolMenubarTk):
         self.setMenuState(ms.hint, "assist.hint")
         self.setMenuState(ms.highlight_piles, "assist.highlightpiles")
         self.setMenuState(ms.find_card, "assist.findcard")
+        self.setMenuState(ms.full_picture, "assist.showfullpicture")
         self.setMenuState(ms.demo, "assist.demo")
         self.setMenuState(ms.demo, "assist.demoallgames")
         # Options menu
@@ -195,6 +218,15 @@ class PysolMenubar(PysolMenubarTk):
         self.setMenuState(ms.autodrop, "options.automaticplay.autodrop")
         self.setMenuState(ms.autodeal, "options.automaticplay.autodeal")
         self.setMenuState(ms.quickplay, "options.automaticplay.quickplay")
+        if USE_PIL:
+            self.setMenuState(ms.autoscale,
+                              "options.cardsize.preserveaspectratio")
+            self.setMenuState(not ms.autoscale,
+                              "options.cardsize.increasethecardsize")
+            self.setMenuState(not ms.autoscale,
+                              "options.cardsize.decreasethecardsize")
+            self.setMenuState(not ms.autoscale,
+                              "options.cardsize.resetthecardsize")
         # Help menu
         self.setMenuState(ms.rules, "help.rulesforthisgame")
         # Toolbar
@@ -203,6 +235,7 @@ class PysolMenubar(PysolMenubarTk):
         self.setToolbarState(ms.undo, "undo")
         self.setToolbarState(ms.redo, "redo")
         self.setToolbarState(ms.autodrop, "autodrop")
+        self.setToolbarState(ms.hint, "hint")
         self.setToolbarState(ms.shuffle, "shuffle")
         self.setToolbarState(ms.pause, "pause")
         self.setToolbarState(ms.rules, "rules")
@@ -251,7 +284,7 @@ class PysolMenubar(PysolMenubarTk):
 
     def _mNewGameBySeed(self, seed, origin):
         try:
-            random = constructRandom(seed)
+            random = construct_random(seed)
             if random is None:
                 return
             id = self.game.id
@@ -311,6 +344,14 @@ class PysolMenubar(PysolMenubarTk):
             if not self.game.areYouSure(_("Select random game")):
                 return
         game_id = None
+        games = self._mGetPossibleRandomGames(type)
+        if games:
+            game_id = self.app.chooseRandomOutOfGames(games)
+        if game_id and game_id != self.game.id:
+            self.game.endGame()
+            self.game.quitGame(game_id)
+
+    def _mGetPossibleRandomGames(self, type='all'):
         games = []
         for g in self.app.gdb.getGamesIdSortedById():
             gi = self.app.getGameInfo(g)
@@ -323,17 +364,15 @@ class PysolMenubar(PysolMenubarTk):
             won, lost = self.app.stats.getStats(self.app.opt.player, gi.id)
             if type == 'all':
                 games.append(gi.id)
+            elif type == 'played' and won + lost > 0:
+                games.append(gi.id)
             elif type == 'won' and won > 0:
                 games.append(gi.id)
             elif type == 'not won' and won == 0 and lost > 0:
                 games.append(gi.id)
-            elif type == 'not played' and won+lost == 0:
+            elif type == 'not played' and won + lost == 0:
                 games.append(gi.id)
-        if games:
-            game_id = self.app.getRandomGameId(games)
-        if game_id and game_id != self.game.id:
-            self.game.endGame()
-            self.game.quitGame(game_id)
+        return games
 
     def _mSelectNextGameFromList(self, gl, step):
         if self._cancelDrag():
@@ -381,7 +420,7 @@ class PysolMenubar(PysolMenubarTk):
         if self._cancelDrag():
             return
         if self.changed():
-            if not self.game.areYouSure(_("Quit ") + TITLE):
+            if not self.game.areYouSure(_("Quit %s") % TITLE):
                 return
         self.game.endGame()
         self.game.quitGame()
@@ -445,7 +484,7 @@ class PysolMenubar(PysolMenubarTk):
         if not self.game.gsaveinfo.bookmarks:
             return
         if not self.game.areYouSure(_("Clear bookmarks"),
-                                    _("Clear all bookmarks ?")):
+                                    _("Clear all bookmarks?")):
             return
         self.game.gsaveinfo.bookmarks = {}
         self.game.updateMenus()
@@ -457,7 +496,7 @@ class PysolMenubar(PysolMenubarTk):
             return
         if self.changed(restart=1):
             if not self.game.areYouSure(_("Restart game"),
-                                        _("Restart this game ?")):
+                                        _("Restart this game?")):
                 return
         self.game.restartGame()
 
@@ -494,6 +533,10 @@ class PysolMenubar(PysolMenubarTk):
             create_find_card_dialog(self.game.top, self.game,
                                     self.app.getFindCardImagesDir())
 
+    def mFullPicture(self, *args):
+        if self.game.canShowFullPicture():
+            create_full_picture_dialog(self.game.top, self.game)
+
     def mSolver(self, *args):
         create_solver_dialog(self.game.top, self.app)
 
@@ -501,10 +544,11 @@ class PysolMenubar(PysolMenubarTk):
         if self._cancelDrag(break_pause=False):
             return
         game, gi = self.game, self.game.gameinfo
-        t = " " + game.getGameNumber(format=1)
-        cc = _("Comments for %s:\n\n") % (gi.name + t)
+        kw = {'game': gi.name,
+              'id': game.getGameNumber(format=1)}
+        cc = _("Comments for %(game)s %(id)s:\n\n") % kw
         c = game.gsaveinfo.comment or cc
-        d = EditTextDialog(game.top, _("Comments for ")+t, text=c)
+        d = EditTextDialog(game.top, _("Comments for %(id)s") % kw, text=c)
         if d.status == 0 and d.button == 0:
             text = d.text
             if text.strip() == cc.strip():
@@ -518,18 +562,17 @@ class PysolMenubar(PysolMenubarTk):
                     text += os.linesep
                 enc = locale.getpreferredencoding()
                 try:
-                    fd = open(fn, 'a')
-                    fd.write(text.encode(enc, 'replace'))
+                    with open(fn, 'at') as fh:
+                        fh.write(text.encode(enc, 'replace'))
                 except Exception as err:
                     d = MfxExceptionDialog(
                         self.top, err,
                         text=_("Error while writing to file"))
                 else:
-                    if fd:
-                        fd.close()
                     d = MfxMessageDialog(
-                        self.top, title=TITLE+_(" Info"), bitmap="info",
-                        text=_("Comments were appended to\n\n") + fn)
+                        self.top, title=_("%s Info") % TITLE, bitmap="info",
+                        text=_("Comments were appended to\n\n%(filename)s")
+                        % {'filename': fn})
         self._setCommentMenu(bool(game.gsaveinfo.comment))
 
     #
@@ -537,34 +580,33 @@ class PysolMenubar(PysolMenubarTk):
     #
 
     def _mStatsSave(self, player, filename, write_method):
-        file = None
         if player is None:
-            text = _("Demo statistics")
+            text = _("Demo statistics were appended to\n\n%(filename)s")
             filename = filename + "_demo"
         else:
-            text = _("Your statistics")
+            text = _("Your statistics were appended to\n\n%(filename)s")
         filename = os.path.join(self.app.dn.config, filename + ".txt")
         filename = os.path.normpath(filename)
         try:
-            file = open(filename, "a")
-            a = FileStatsFormatter(self.app, file)
+            a = FileStatsFormatter(self.app, open(filename, "a"))
             write_method(a, player)
         except EnvironmentError as ex:
-            if file:
-                file.close()
             MfxExceptionDialog(self.top, ex,
                                text=_("Error while writing to file"))
         else:
-            if file:
-                file.close()
             MfxMessageDialog(
-                self.top, title=TITLE+_(" Info"), bitmap="info",
-                text=text + _(" were appended to\n\n") + filename)
+                self.top, title=_("%s Info") % TITLE, bitmap="info",
+                text=text % {'filename': filename})
 
     def mPlayerStats(self, *args, **kw):
+        wasPaused = False
+        if not self.game.pause:
+            self.game.doPause()
+            wasPaused = True
         mode = kw.get("mode", 101)
         demo = 0
         gameid = None
+        gamenum = None
         while mode > 0:
             if mode > 1000:
                 demo = not demo
@@ -573,31 +615,44 @@ class PysolMenubar(PysolMenubarTk):
             d = Struct(status=-1, button=-1)
             if demo:
                 player = None
-                p0, p1, p2 = TITLE+_(" Demo"), TITLE+_(" Demo "), ""
             else:
                 player = self.app.opt.player
-                p0, p1, p2 = player, "", _(" for ") + player
             n = self.game.gameinfo.name
+            # translation keywords
+            transkw = {'app': TITLE,
+                       'player': player,
+                       'game': n,
+                       'tops': TOP_SIZE}
             #
             if mode == 100:
                 d = Status_StatsDialog(self.top, game=self.game)
             elif mode == 101:
-                header = p1 + _("Statistics for ") + n
+                header = (_("%(app)s Demo Statistics for %(game)s") if demo
+                          else _("Statistics for %(game)s")) % transkw
                 d = SingleGame_StatsDialog(
                    self.top, header, self.app, player, gameid=self.game.id)
                 gameid = d.selected_game
             elif mode == 102:
-                header = p1 + _("Statistics") + p2
+                header = (_("%(app)s Demo Statistics") if demo
+                          else _("Statistics for %(player)s")) % transkw
                 d = AllGames_StatsDialog(self.top, header, self.app, player)
                 gameid = d.selected_game
             elif mode == 103:
-                header = p1 + _("Full log") + p2
+                header = (_("%(app)s Demo Full log") if demo
+                          else _("Full log for %(player)s")) % transkw
                 d = FullLog_StatsDialog(self.top, header, self.app, player)
+                gameid = d.selected_game
+                gamenum = d.selected_game_num
             elif mode == 104:
-                header = p1 + _("Session log") + p2
+                header = (_("%(app)s Demo Session log") if demo
+                          else _("Session log for %(player)s")) % transkw
                 d = SessionLog_StatsDialog(self.top, header, self.app, player)
+                gameid = d.selected_game
+                gamenum = d.selected_game_num
             elif mode == 105:
-                header = p1 + TOP_TITLE + _(" for ") + n
+                # TRANSLATORS: eg. top 10 or top 5 results for a certain game
+                header = (_("%(app)s Demo Top %(tops)d for %(game)s") if demo
+                          else _("Top %(tops)d for %(game)s")) % transkw
                 d = Top_StatsDialog(
                     self.top, header, self.app, player, gameid=self.game.id)
             elif mode == 106:
@@ -623,7 +678,8 @@ class PysolMenubar(PysolMenubarTk):
                 # reset all player stats
                 if self.game.areYouSure(
                     _("Reset all statistics"),
-                    _("Reset ALL statistics and logs for player\n%s ?") % p0,
+                    _("Reset ALL statistics and logs for player\n" +
+                      "%(player)s?") % transkw,
                     confirm=1, default=1
                 ):
                     self.app.stats.resetStats(player, 0)
@@ -633,8 +689,8 @@ class PysolMenubar(PysolMenubarTk):
                 # reset player stats for current game
                 if self.game.areYouSure(
                     _("Reset game statistics"),
-                    _('Reset statistics and logs ' +
-                        'for player\n%s\nand game\n%s ?') % (p0, n),
+                    _('Reset statistics and logs for player\n%(player)s\n'
+                      'and game\n%(game)s?') % transkw,
                     confirm=1, default=1
                 ):
                     self.app.stats.resetStats(player, self.game.id)
@@ -647,14 +703,21 @@ class PysolMenubar(PysolMenubarTk):
                     self.game.quitGame(gameid)
             elif mode == 402:
                 # start a new game with a gameid / gamenumber
-                # TODO
-                pass
+                if (gameid and gamenum and
+                        (gameid != self.game.id or
+                         gamenum != self.game.getGameNumber(format=0))):
+                    self.game.endGame()
+                    self.game.quitGame(gameid,
+                                       random=construct_random(gamenum))
             else:
                 print_err("stats problem: %s %s %s" % (mode, demo, player))
                 pass
             if d.status != 0:
                 break
             mode = d.button
+        if self.game.pause:
+            if wasPaused:
+                self.game.doPause()
 
     #
     # Assist menu
@@ -722,6 +785,7 @@ class PysolMenubar(PysolMenubarTk):
                 self.game.updateStatus(player=self.app.opt.player)
                 self.game.updateStatus(stats=self.app.stats.getStats(
                     self.app.opt.player, self.game.id))
+        self.updateMenus()
 
     def mOptColors(self, *args):
         if self._cancelDrag(break_pause=False):
@@ -795,6 +859,11 @@ class PysolMenubar(PysolMenubarTk):
         # FIXME: plugins
         help_html(self.app, self.app.getGameRulesFilename(self.game.id), dir)
 
+    def mHelpReportBug(self, *args):
+        if self._cancelDrag(break_pause=False):
+            return
+        help_html(self.app, "report_bug.html", "html")
+
     def mHelpLicense(self, *args):
         if self._cancelDrag(break_pause=False):
             return
@@ -837,7 +906,8 @@ class PysolMenubar(PysolMenubarTk):
     def mPlayNextMusic(self, *args):
         if self._cancelDrag(break_pause=False):
             return
-        if self.app.audio and self.app.opt.sound_music_volume > 0:
+        if (self.app.audio and self.app.opt.music and
+                self.app.opt.sound_music_volume > 0):
             self.app.audio.playNextMusic()
             if 1 and DEBUG:
                 index = self.app.audio.getMusicInfo()
@@ -911,6 +981,11 @@ class PysolToolbar(PysolToolbarTk):
             self.menubar.mShuffle()
         return 1
 
+    def mHint(self, *args):
+        if not self._busy():
+            self.menubar.mHint()
+        return 1
+
     def mPause(self, *args):
         if not self._busy():
             self.menubar.mPause()
