@@ -37,8 +37,6 @@ from pysollib.stack import \
         InitialDealTalonStack
 from pysollib.util import ANY_SUIT
 
-from six.moves import range
-
 
 class Shisen_Hint(AbstractHint):
     TOP_MATCHING = False
@@ -93,6 +91,7 @@ class Shisen_Foundation(AbstractFoundationStack):
 
 
 class Shisen_RowStack(Mahjongg_RowStack):
+    allowAdjacent = True
 
     def basicIsBlocked(self):
         return 0
@@ -112,7 +111,9 @@ class Shisen_RowStack(Mahjongg_RowStack):
             a.append([5]*(rows+2))
 
         def can_move(x, y, nx, ny, direct, d, direct_chng_cnt):
-            if nx == x2 and ny == y2:
+            if (nx == x2 and ny == y2 and (self.allowAdjacent or
+                                           (abs(dx) > 1 and abs(dy) > 1)
+                                           or direct_chng_cnt > 0)):
                 return 1
             if nx < 0 or ny < 0 or nx > cols+1 or ny > rows+1:
                 return 0
@@ -250,9 +251,10 @@ class Shisen_RowStack(Mahjongg_RowStack):
         cs = game.app.cardset
         path = self.acceptsCards(other_stack, [other_stack.cards[-1]])
         # print path
-        x0, y0 = game.XMARGIN, game.YMARGIN
+        x0, y0 = (game.XMARGIN + game.center_offset[0],
+                  game.YMARGIN + game.center_offset[1])
         cardw, cardh = images.CARDW, images.CARDH
-        if cs.version >= 6:
+        if cs.version == 6 or cs.mahjongg3d:
             cardw -= cs.SHADOW_XOFFSET
             cardh -= cs.SHADOW_YOFFSET
         coords = []
@@ -309,8 +311,8 @@ class AbstractShisenGame(AbstractMahjonggGame):
         l, s = Layout(self), self.s
         # dx, dy = 3, -3
 
-        cs = self.app.cardset
-        if cs.version >= 6:
+        cs = self.app.images.cs
+        if cs.version == 6 or cs.mahjongg3d:
             dx = l.XOFFSET
             dy = -l.YOFFSET
             d_x = cs.SHADOW_XOFFSET
@@ -322,6 +324,10 @@ class AbstractShisenGame(AbstractMahjonggGame):
             d_x = 0
             d_y = 0
             self._delta_x, self._delta_y = 0, 0
+        # TODO - This should be moved to subsample logic in the future.
+        if self.preview > 1:
+            d_x /= 2
+            d_y /= 2
 
         font = self.app.getFont("canvas_default")
 
@@ -497,6 +503,50 @@ class NotShisen_24x12(AbstractShisenGame):
     NCARDS = 288
 
 
+# ************************************************************************
+# * Four Rivers
+# ************************************************************************
+class FourRivers_RowStack(Shisen_RowStack):
+    allowAdjacent = False
+
+
+class FourRivers_14x6(AbstractShisenGame):
+    RowStack_Class = FourRivers_RowStack
+    L = (14, 6)
+    NCARDS = 84
+
+
+class FourRivers_18x8(AbstractShisenGame):
+    RowStack_Class = FourRivers_RowStack
+    L = (18, 8)
+
+
+class FourRivers_24x12(AbstractShisenGame):
+    RowStack_Class = FourRivers_RowStack
+    L = (24, 12)
+    NCARDS = 288
+
+
+class FourRivers_14x6_NoGravity(AbstractShisenGame):
+    RowStack_Class = FourRivers_RowStack
+    L = (14, 6)
+    NCARDS = 84
+    GRAVITY = False
+
+
+class FourRivers_18x8_NoGravity(AbstractShisenGame):
+    RowStack_Class = FourRivers_RowStack
+    L = (18, 8)
+    GRAVITY = False
+
+
+class FourRivers_24x12_NoGravity(AbstractShisenGame):
+    RowStack_Class = FourRivers_RowStack
+    L = (24, 12)
+    NCARDS = 288
+    GRAVITY = False
+
+
 # ************************************************************************
 # * register a Shisen-Sho type game
 # ************************************************************************
@@ -524,6 +574,14 @@ r(11006, Shisen_24x12_NoGravity, "Shisen-Sho (No Gravity) 24x12")
 r(11011, NotShisen_14x6, "Not Shisen-Sho 14x6", "notshisensho.html")
 r(11012, NotShisen_18x8, "Not Shisen-Sho 18x8", "notshisensho.html")
 r(11013, NotShisen_24x12, "Not Shisen-Sho 24x12", "notshisensho.html")
-
+r(11014, FourRivers_14x6, "Four Rivers 14x6", "fourrivers.html")
+r(11015, FourRivers_18x8, "Four Rivers 18x8", "fourrivers.html")
+r(11016, FourRivers_24x12, "Four Rivers 24x12", "fourrivers.html")
+r(11017, FourRivers_14x6_NoGravity, "Four Rivers (No Gravity) 14x6",
+  "fourrivers.html")
+r(11018, FourRivers_18x8_NoGravity, "Four Rivers (No Gravity) 18x8",
+  "fourrivers.html")
+r(11019, FourRivers_24x12_NoGravity, "Four Rivers (No Gravity) 24x12",
+  "fourrivers.html")
 
 del r
