@@ -23,6 +23,7 @@
 
 import os
 import time
+import tkinter
 
 from pysollib.mfxutil import KwStruct, kwdefault
 from pysollib.mfxutil import format_time
@@ -31,7 +32,6 @@ from pysollib.settings import TOP_TITLE
 from pysollib.stats import ProgressionFormatter, PysolStatsFormatter
 from pysollib.ui.tktile.tkutil import bind, loadImage
 
-from six.moves import tkinter
 from six.moves import tkinter_font
 
 from .tkwidget import MfxDialog, MfxMessageDialog
@@ -410,7 +410,8 @@ class CanvasFormatter(PysolStatsFormatter):
         #
         y += self.h
         total, played, won, lost, time_, moves, perc = self.getStatSummary()
-        s = _("Total (%d out of %d games)") % (played, total)
+        s = _("Total (%(played)d out of %(total)d games)") % {
+            'played': played, 'total': total}
         self.pstats(y, (s, won+lost, won, lost, time_, moves, perc))
 
     def writeLog(self, player, prev_games):
@@ -486,7 +487,12 @@ class AllGames_StatsDialog(MfxDialog):
         self.nodes = {}
         self.canvas = self.sc.canvas
         self.canvas.dialog = self
-        bind(self.canvas, "<1>", self.singleClick)
+        from pysollib.options import calcCustomMouseButtonsBinding
+        bind(
+            self.canvas,
+            calcCustomMouseButtonsBinding("<{mouse_button1}>"),
+            self.singleClick
+        )
         self.fillCanvas(player, title)
         bbox = self.canvas.bbox("all")
         # print bbox
@@ -608,7 +614,8 @@ class Status_StatsDialog(MfxMessageDialog):
         w1 = (_("Highlight piles: ") + str(stats.highlight_piles) + "\n" +
               _("Highlight cards: ") + str(stats.highlight_cards) + "\n" +
               _("Highlight same rank: ") + str(stats.highlight_samerank) +
-              "\n")
+              "\n" +
+              _('Face-down peeks: ') + str(stats.peeks) + '\n')
         if game.s.talon:
             if game.gameinfo.redeals != 0:
                 w2 = w2 + _("\nRedeals: ") + str(game.s.talon.round - 1)
@@ -771,8 +778,8 @@ class Top_StatsDialog(MfxDialog):
             #                 s.score_casino_result.min,
             #                 s.score_casino_result.max,
             #                 round(s.score_casino_result.average, 2), ))
-            for l, min, max, avr, tot, top in ll:
-                tkinter.Label(frame, text=l).grid(row=row, column=0)
+            for label, min, max, avr, tot, top in ll:
+                tkinter.Label(frame, text=label).grid(row=row, column=0)
                 tkinter.Label(frame, text=str(min)).grid(row=row, column=1)
                 tkinter.Label(frame, text=str(max)).grid(row=row, column=2)
                 tkinter.Label(frame, text=str(avr)).grid(row=row, column=3)
